﻿// references Microsoft.SPOT.Native, Microsoft.SPOT.Net, mscorlib, System
using System.Net;
using System.Net.Sockets;
using System.Text;
using Microsoft.SPOT;
using Microsoft.SPOT.Net.NetworkInformation;

public class GsiotDiagnostics
{
    public static void Main()
    {
        Debug.Print("Diagnostics started");
        Debug.Print("Version 30-Jun-2011");

        NetworkInformation();

        // try to reach some server on the Internet, e.g., Google
        TryGoogle();

        Debug.GC(true);

        // try to connect to Yaler
        TryYaler();

        Debug.Print("Diagnostics completed");
    }

    static void NetworkInformation()
    {
        NetworkInterface[] nis = NetworkInterface.GetAllNetworkInterfaces();
        if (nis[0] == null)
        {
            Debug.Print("  ERROR: no network interface found");
        }

        var x = new char[17];
        var i = 0;
        byte[] a = nis[0].PhysicalAddress;
        const string hex = "0123456789ABCDEF";
        for (var j = 0; j != 6; j = j + 1)
        {
            x[i] = (char)(hex[(a[j] & 0xF0) >> 4]);
            i = i + 1;
            x[i] = (char)(hex[a[j] & 0x0F]);
            i = i + 1;
            if (i != 17)
            {
                x[i] = '-';
                i = i + 1;
            }
        }
        string mac = new string(x);

        Debug.Print("  DHCP enabled: " + nis[0].IsDhcpEnabled);
        Debug.Print("  MAC address: " + mac);
        string localHost = nis[0].IPAddress;
        Debug.Print("  Device address: " + localHost);
        Debug.Print("  Gateway address: " + nis[0].GatewayAddress);
        if (nis[0].DnsAddresses.Length > 0)
        {
            Debug.Print("  Primary DNS address: " + nis[0].DnsAddresses[0]);
        }
    }

    static void TryGoogle()
    {
        Debug.Print("Trying to connect to www.google.com...");
        using (Socket s = Connect("www.google.com", 5000))
        {
            var request = "HEAD / HTTP/1.1\r\nHost: www.google.com\r\n\r\n";
            byte[] requestBuffer = Encoding.UTF8.GetBytes(request);
            Debug.Print("  sending request");
            s.Send(requestBuffer);
                // if a SocketException with ErrorCode 10053 occurred: firewall problem?

            Debug.Print("  receiving response (can take a minute or so!)...");
            Receive(s);
            Debug.Print("  close connection");
            s.Close();
            Debug.Print("  connection to Google was successful");
        }
    }

    static void TryYaler()
    {
        Debug.Print("Trying to connect to try.yaler.net...");
        using (Socket s = Connect("try.yaler.net", 5000))
        {
            var request = "HEAD /ping HTTP/1.1\r\nHost: try.yaler.net\r\n\r\n";
            byte[] requestBuffer = Encoding.UTF8.GetBytes(request);
            Debug.Print("  sending request");
            s.Send(requestBuffer);
            Debug.Print("  receiving response (can take a minute or so!)...");
            Receive(s);
            Debug.Print("  close connection");
            s.Close();
            Debug.Print("  connection to Yaler was successful");
        }
    }

    static Socket Connect(string host, int timeout)
    {
        Debug.Print("  in Connect");
        // look up host's domain name to find IP address(es)
        Debug.Print("    resolving the host name (can take a minute or so!)...");
        IPHostEntry hostEntry;
        try
        {
            hostEntry = Dns.GetHostEntry(host);
        }
        catch (SocketException)
        {
            Debug.Print("    ERROR: host name could not be resolved");
            Debug.Print("      -> are all cables connected?");
            Debug.Print("      -> do you have a PC on the same local network whose Web browser works?");
            return null;
        }

        // extract a returned address
        IPAddress hostAddress = hostEntry.AddressList[0];
        Debug.Print("    creating new IP endpoint");
        IPEndPoint remoteEndPoint = new IPEndPoint(hostAddress, 80);

        // connect!
        var connection = new Socket(AddressFamily.InterNetwork,
            SocketType.Stream, ProtocolType.Tcp);
        Debug.Print("    connecting...");
        connection.Connect(remoteEndPoint);
        connection.SetSocketOption(SocketOptionLevel.Tcp,
            SocketOptionName.NoDelay, true);
        connection.SendTimeout = timeout;
        if (connection == null)
        {
            Debug.Print("    ERROR: no connection could be established");
        }
        else
        {
            Debug.Print("    connection is established");
        }
        return connection;
    }

    static void Receive(Socket s)
    {
        char[] x = new char[3];
        byte[] b = new byte[1];

        for (int j = 0; j != 12; j = j + 1)
        {
            int n = s.Receive(b);
            x[j % 3] = n != 0 ? (char)b[0] : (char)0;
        }
        Debug.Print("    12 bytes received");

        if ((x[0] == '2') && (x[1] == '0') && (x[2] == '0'))
        {
            Debug.Print("    status code = 200 (OK)");
        }
        else if ((x[0] == '3') && (x[1] == '0') && (x[2] == '2'))
        {
            Debug.Print("    status code = 302 (Found)");
        }
        else if ((x[0] == '5') && (x[1] == '0') && (x[2] == '4'))
        {
            Debug.Print("    status code = 504 (Gateway Timeout, this is the Yaler response to an unknown relay domain)");
        }
        else
        {
            Debug.Print("    ERROR: unexpected status code");
            Debug.Print(x[0].ToString());
            Debug.Print(x[1].ToString());
            Debug.Print(x[2].ToString());
        }
    }
}
